# ***** BEGIN LICENSE BLOCK *****
# * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
# ***** END LICENSE BLOCK *****

###
### Functions to work with lilo.conf
###

. shell-error
. shell-quote
. alterator-hw-functions

device_exists() {
	[ ! -e "$1" ] || return 0
	write_error "$1: `_ "device not found"`"
	return 1
}

cmdline_params() {
	local vga= args= new_args=

	vga="$(sed -rne 's|^(.* )?vga=([^ ]+)( .*)?$|\2|p' /proc/cmdline)"
	[ -z "$vga" ] || replace_global 'vga' "vga=\"$vga\""

	for p in $(cat /proc/cmdline); do
		case "$p" in
			# ACPI/APM
			acpi=*|noapic|acpi_*|nolapic*|lapic*|no_timer_check|pnpacpi=*|apm=*) ;;

			# [LIBATA] DMA control
			libata.*) ;;

			# [MOUSE]
			mousedev.*) ;;

			# [HW] (E)IDE subsystem
			# See Documentation/ide.txt
			ide=*|idebus=*) ;;

			# [BUGS=X86-32]
			mem=*|nmi_watchdog=*|no387|notsc) ;;

			# [PCI] various PCI subsystem options
			pci=*) ;;

			# Misc
			idle=*|irqfixup|irqpoll|clocksource=*|console=*|splash=*|quiet*) ;;

			*) continue ;;
		esac
		new_args="$new_args $p"
	done
	args="$(show_global_param 'append')"
	args="$args${new_args:+ $new_args}"
	args="${args# }"

	[ -z "$args" ] ||
		replace_global 'append' "append=\"$args\""

	! readlink -e '/boot/splash/message' >/dev/null ||
		replace_global 'message' "message=\"/boot/splash/message\""

	commit "$globals"
}

update_rootdev() {
        local cnf="$1"
        local rootdev="$(get_part_with_mntpt "/")"
        local uuid="$(/sbin/blkid -s UUID "$rootdev")"
        uuid="${uuid##*UUID=\"}"
        uuid="${uuid%%\"*}"
        local quuid
        quote_sed_regexp_variable quuid "$uuid"
        sed -i -e "s,^root=.*,root=\"UUID=$quuid\"," "$cnf"
}

write_liloconf() {
	local tmpconf="$1";  shift
	local newconf="$1";  shift
	local liloopts="$1"; shift

	local bootdev="$(sed -rne 's,^boot=\"?([^\"]+)\"?,\1,p' "$tmpconf")"
	if [ -z "$bootdev" ]; then
		write_error "`_ "No boot device"`"
		return 1
	fi

	[ "$bootdev" != "${bootdev#/}" ] || bootdev="/dev/$bootdev"
	device_exists "$bootdev" || return 1

	# Test lilo.conf
	
	local testprg="/sbin/lilo -t $liloopts -C \"$tmpconf\" </dev/null 2>&1 >/dev/null"
	if [ -n "$LILO_DESTDIR" ]; then
	    install -Dpm644 "$tmpconf" "$LILO_DESTDIR/$tmpconf"
	    testprg="chroot "$LILO_DESTDIR" $testprg"
	fi

	local E01 E02 E03 E04 E05 E06 E07 E08 E09 E10 E11 E12 E13 E14 E15
	local E01L E02L E03L E04L E05L E06L E07L E08L E09L E10L E11L E12L E13L E14L E15L

	quote_sed_regexp_variable E01 "Not a RAID install, 'raid-extra-boot=' not allowed"
	quote_sed_regexp_variable E02 "RAID1 install implied by 'boot=/'"
	quote_sed_regexp_variable E03 "Unable to open"
	quote_sed_regexp_variable E04 "Unable to stat"
	quote_sed_regexp_variable E05 "is not a block device"
	quote_sed_regexp_variable E06 "Unable to get RAID version on"
	quote_sed_regexp_variable E07 "Raid major versions > 0 are not supported"
	quote_sed_regexp_variable E08 "Raid versions < 0.90 are not supported"
	quote_sed_regexp_variable E09 "Unable to get RAID info on"
	quote_sed_regexp_variable E10 "Inconsistent Raid version information on"
	quote_sed_regexp_variable E11 "Only RAID1 devices are supported as boot devices"
	quote_sed_regexp_variable E12 "RAID install requires LBA32 or LINEAR; LBA32 assumed."
	quote_sed_regexp_variable E13 "boot device not found"
	quote_sed_regexp_variable E14 "device not found"
	quote_sed_regexp_variable E15 "Could not find kernel image"

	quote_sed_regexp_variable E01L "`_ "Not a RAID install, 'raid-extra-boot=' not allowed"`"
	quote_sed_regexp_variable E02L "`_ "RAID1 install implied by 'boot=/'"`"
	quote_sed_regexp_variable E03L "`_ "Unable to open"`"
	quote_sed_regexp_variable E04L "`_ "Unable to stat"`"
	quote_sed_regexp_variable E05L "`_ "is not a block device"`"
	quote_sed_regexp_variable E06L "`_ "Unable to get RAID version on"`"
	quote_sed_regexp_variable E07L "`_ "Raid major versions > 0 are not supported"`"
	quote_sed_regexp_variable E08L "`_ "Raid versions < 0.90 are not supported"`"
	quote_sed_regexp_variable E09L "`_ "Unable to get RAID info on"`"
	quote_sed_regexp_variable E10L "`_ "Inconsistent Raid version information on"`"
	quote_sed_regexp_variable E11L "`_ "Only RAID1 devices are supported as boot devices"`"
	quote_sed_regexp_variable E12L "`_ "RAID install requires LBA32 or LINEAR; LBA32 assumed."`"
	quote_sed_regexp_variable E13L "`_ "boot device not found"`"
	quote_sed_regexp_variable E14L "`_ "device not found"`"
	quote_sed_regexp_variable E15L "`_ "Could not find kernel image"`"
	# todo: add more messages to localize

	local out_convs="
	  s/$E01/$E01L/
	  s/$E02/$E02L/
	  s/$E03/$E03L/
	  s/$E04/$E04L/
	  s/$E05/$E05L/
	  s/$E06/$E06L/
	  s/$E07/$E07L/
	  s/$E08/$E08L/
	  s/$E09/$E09L/
	  s/$E10/$E10L/
	  s/$E11/$E11L/
	  s/$E12/$E12L/
	  s/$E13/$E13L/
	  s/$E14/$E14L/
	  s/$E15/$E15L/
	"

	if ! out="$(eval $testprg)"; then
	        out="$(echo "$out" | sed -e "$out_convs")"
		write_error "Lilo test: $out"
		openclose_cdrom
		return 1
	fi
	verbose "write_liloconf(): lilo test passed"

	# Set bootable flag
	set_bootable_flag $(blockdev_follow_symlink "$bootdev")

	# Write lilo.conf
	[ ! -f "$newconf" ] || mv -f -- "$newconf" "$newconf".save
	mv -f -- "$tmpconf" "$newconf"

	# Real install bootloader
	write_debug "Executing lilo %s\n" "$liloopts"
	if [ -z "$LILO_DESTDIR" ]; then
	  /sbin/lilo $liloopts
	else
	  chroot "$LILO_DESTDIR" /sbin/lilo $liloopts
	fi
	openclose_cdrom
}

generate_liloconf() {
	tmpconf="$1"; shift

	install -m600 "$globals" "$tmpconf"

	while read -r t s; do
		[ -f "$sections/$s" ] || continue
		printf \\n
		format_section "$sections/$s"
	done < "$order" >> "$tmpconf"
}

normalize_liloconf() {
	local keyword dummy sysconf= tmpconf= conflist=
	sysconf="$1"; shift
	tmpconf="$1"; shift

	if [ -s "$sysconf" ]; then
		grep -qs '^boot=' "$sysconf" &&
			conflist="$sysconf" ||
			conflist="$tmpconf $sysconf"
	else
		conflist="$tmpconf"
	fi

	sed -r	-e 's#[[:space:]]+# #g' \
		-e 's#^[[:space:]]*([^=]+=)(.*)#\1 \2#' \
		$conflist |
	while read -r keyword dummy; do
		[ -n "$keyword" ] || continue
		if [ -z "${keyword##*=}" -a -n "$dummy" ]; then
			[ -z "${dummy%%\"*}" ] || dummy="\"$dummy"
			[ -z "${dummy##*\"}" ] || dummy="$dummy\""
		fi
		keyword="$(printf %s\\n "$keyword" |tr '[A-Z]' '[a-z]')"
		printf %s\\n "$keyword$dummy"
	done
}
