#!/bin/bash

# functions for getting hardware information for alterator-lilo
# * places for bootloader (list_bootloader_places, get_default_boot_dev)
# * other OS (find_others)
# * ...


# for localized lists set
# dec_pt=$(LC_NUMERIC="$in_language" locale decimal_point)

. alterator-sh-functions
. alterator-hw-functions

quote_udevadm_out='s|^\([^=]\+\)=\(.*\)|\1="\2"|'

LILO_DESTDIR="${LILO_DESTDIR:-}"

list_bootloader_places(){

  if [ -n "${raidboot:-}" ]; then
    echo "$raidboot   RAID (${raidboot#/dev/})"
    return 0
  fi

  for i in $(disk_list); do

    ! disk_is_virtual   "$i" || continue
    ! disk_is_removable "$i" || continue

    local info="$(disk_info "$i")"

    unset DEVNAME
    eval $(/sbin/udevadm info --name=$i --query=env 2>/dev/null | sed -e "$quote_udevadm_out")

    local size="$(disk_size "$i")"
    size="$(human_readable_size "$size")"

    DEVNAME="${DEVNAME:-/dev/$i}" # on 2.6.24 udevadm will fail
    local sym="$(blockdev_get_symlink "${DEVNAME:-/dev/$i}")"

    echo "$sym   Hard drive ${DEVNAME##*/}${info:+, $info}${size:+, $size}"
  done

  local bootdev="$(get_part_with_mntpt "/boot")"
  local rootdev="$(get_part_with_mntpt "/")"
  local dev=${bootdev:-$rootdev}
  [ -z "$dev" ] || echo "$(blockdev_get_symlink $dev)   Linux partition (${dev#/dev/})"
}

# find other loaders in system (on partitions only!)
# (for creation "other" sections in lilo.conf)
find_others(){

  # Exclude self partitions
  exclude="$(get_part_with_mntpt "/")
           $(get_part_with_mntpt "/boot")
           $(get_part_with_mntpt "/usr")
           $(get_part_with_mntpt "/home")
           $(get_part_with_mntpt "/var")
           "
  local dev win_n lin_n label

  for dev in $(partition_list | partition_sort) ; do

    [ "${exclude##*$dev}" = "$exclude" ] || continue

    local type="$(partition_id $dev)"
    local fstype="$(partition_fstype $dev)"

    [ "$fstype" = 'ext2' -o "$fstype" = 'ext3' -o "$fstype" = 'reiserfs' ] && type="${type:-83}"
    [ "$fstype" = 'ntfs' -o "$fstype" = 'vfat' -o "$fstype" = 'fat' ]      && type="${type:-b}"

    [ -n "$type" ] || continue
    bootsector "/dev/$dev" &>/dev/null || continue

    case "$type" in
      1|4|6|7|b|c|e|11|14|16|17|1b|1c|1e|86|87|c1|c4|c6|ef)
        label="Windows${win_n:+$win_n}"
        [ -z "$win_n" ] && win_n=0
        win_n=$((win_n+1))
      ;;
      81|83|85|88|8e|f0|fd)
        label="Linux${lin_n:+$lin_n}"
        [ -z "$lin_n" ] && lin_n=0
        lin_n=$((lin_n+1))
      ;;
      2|3) label="Xenix"    ;;
      8|9) label="Aix"      ;;
      a)   label="OS/2"     ;;
      39)  label="Plan9"    ;;
      a5)  label="FreeBSD"  ;;
      a6)  label="OpenBSD"  ;;
      a9)  label="NetBSD"   ;;
      ab)  label="Darwin"   ;;
      63)  label="HURD"     ;;
      be)  label="Solaris"  ;;
      eb)  label="BeOS"     ;;
      #... add more
      *)  continue ;;
    esac
    # 1. device (from /dev/disks/by-id/*)
    # 2. short label (without spaces - for lilo.conf)
    echo $(blockdev_get_symlink $dev) $label
  done
}


# get devices of partition with given mntpt
# in installer we need /dev/sda1 instead of /dev/evms/sda1,
# /dev/md0 instead of /dev/evms/md/md0
#
# ways to make this substitution:
#  hal, udevadm -- very unstable with our evms
#  blkid -- don't work with raids
#
# so we use rather straight method

get_part_with_mntpt(){
  local target="$1"

  local dev mntpt type opts n1 n2
  while read dev mntpt type opts n1 n2; do

    [ "$mntpt" = "$target" ] || continue
    [ "$type" != "rootfs" -a "$type" != "squashfs" ] || continue

    if [ "${dev#UUID=}" != "$dev" ]; then
      for d in $(/sbin/blkid -t "$dev" -s UUID | cut -d' ' -f1); do
        d="${d%:}"
        ! disk_is_virtual "${d#/dev/}" || [ -z "${d##/dev/md*}" ] || continue
        dev="$d"
        break
      done
    fi

    dev="$(readlink -f "$dev")"

    local noevms="${dev#/dev}"
    noevms="${noevms#/evms/md}"
    noevms="/dev${noevms#/evms}"

    [ -b "$noevms" ] && echo "$noevms" || echo "$dev"

  done < "$LILO_DESTDIR/etc/fstab"
}

# for cdrom openclose
# cdroms on ide bus only (!?)
get_ide_cdroms(){
  [ -d /sys/bus/ide/devices ] || return 0
  for d in $(ls /sys/bus/ide/devices); do
    DIR="/sys/bus/ide/devices/$d"
    if [ -f "$DIR/media" -a "$(cat $DIR/media)" = "cdrom" -a\
      -f "$DIR/drivename" ]; then
      echo -n "/dev/"
      cat "$DIR/drivename" ||:
    fi
  done
}

# get_default_boot_dev() -- find recommended device for bootloader.
# It is raid device in the case of root on raid, or boot disk found in
# bios or just first hard drive

get_bios_boot(){
  local bios_dev="$(/sbin/lilo -T bios |\
     sed -n 's/BIOS provided boot device is \([^ ]\+\) .*/\1/p')"
  [ -n "$bios_dev" ] || return
  local vol_id="$(/sbin/lilo -T vol-ID |\
     sed -n -e "s/^[[:space:]]\+$bios_dev[[:space:]]\+\([[:alnum:]]\+\)[[:space:]]*$/\1/p")"
  [ -n "$vol_id" ] || return

  local dev model
  list_bootloader_places | while read dev model; do
    lilo -T table="$dev" | grep -q $vol_id || continue
    blockdev_get_symlink "$dev"
    return
  done
}
get_default_boot_dev(){
  if [ -n "${raidboot:-}" ]; then
    echo "$raidboot"
    return
  fi
  local bios_boot="$(get_bios_boot)"
  local first_disk="$((list_bootloader_places) | head -n1 | cut -d' ' -f1)" 
  # We need to run list_bootloader_places in subshell to avoid error messages
  # from the shell.
  # Head closes pipe, so somebody get SIGPIPE. Without subshell it is our
  # main shell and it kills echo command with error message. If subshell
  # exists it will be silently killed.
  echo "${bios_boot:-$first_disk}"
}

# ensure boot device to be in results of list_bootloader_places()
fix_boot(){
  bootdev="$(blockdev_get_symlink ${1:-})"
  list_bootloader_places | (
    dev="$(get_default_boot_dev)"
    while read d l; do
      if [ "$bootdev" = "$d" ]; then
        dev="$d"
        break
      fi
    done
    echo "$dev"
  )
}


